﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Event: DateChanged (Value As Long)
#Event: Folding (Value As Boolean)
#DesignerProperty: Key: BackgroundColor, DisplayName: Background Color, FieldType: Color, DefaultValue: 0xFF000000, Description: Canvas background color.
#DesignerProperty: Key: ControlFont, DisplayName: Font, FieldType: String, DefaultValue: ZonaPro-Thin.otf, Description: The font to use.
#DesignerProperty: Key: ControlFontColor, DisplayName: Font Color, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Font color to use for labels
Sub Class_Globals
	Private EventName As String 'ignore
	Private CallBack As Object 'ignore
	Private Canvas As Panel
	Private LblDate As Label
	Private IVDownArrow As ImageView
	Private DateFormatStr As String
	
	Private LblMonth As Label
	Private IVLeftArrowMonth As ImageView
	Private IVRightArrowMonth As ImageView
	
	Private LblDay As Label
	Private IVLeftArrowDay As ImageView
	Private IVRightArrowDay As ImageView	
	
	Private CurrentStamp As Long
	Private Unfolded As Boolean = False
	Private InitialHeight As Int
	Private UnfoldedHeight As Int
	Public CurrentHeight As Int
	
	Dim ControlHeight As Int = 30dip
	Dim ControlGapV As Int = 10dip	
	Dim MyTimer As Timer
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	EventName = vEventName
	CallBack = vCallback
	Unfolded = False
End Sub

Public Sub DesignerCreateView (Base As Panel, Lbl As Label, Props As Map)
	Canvas = Base
	Canvas.Color = Props.Get("BackgroundColor")
	
	Dim CurrentY As Int = 0
	DateFormatStr = CDDatePickerStorage.DateFormat
	DateTime.DateFormat = DateFormatStr
	
	CurrentStamp = DateTime.Now

	' Add Label to Show Current Date
	LblDate.Initialize("LblDate")
	LblDate.TextSize = CDDatePickerStorage.FontSize
	LblDate.TextColor = Props.Get("ControlFontColor")
	LblDate.Text = DateTime.Date(CurrentStamp)
	LblDate.Gravity = Gravity.CENTER
	
	Canvas.AddView(LblDate, 0, 0, Canvas.Width - 35dip, ControlHeight)
	
	InitialHeight = ControlHeight
	Canvas.Height = InitialHeight
	
	IVDownArrow.Initialize("IVImageView")
	Canvas.AddView(IVDownArrow, Canvas.Width - 30dip, 0, ControlHeight, ControlHeight)
	RenderSVG(IVDownArrow, "i-down-arrow.svg", 1)
	
	CurrentY = CurrentY + ControlHeight + ControlGapV
		
	' DAY
	' Add Label to Show Current Date
	LblDay.Initialize("LblDay")
	LblDay.Text = GetDayString(CurrentStamp)
	LblDay.TextSize = CDDatePickerStorage.FontSize
	LblDay.Gravity = Gravity.CENTER
	LblDay.TextColor = Props.Get("ControlFontColor")
	
	Canvas.AddView(LblDay, 35dip, CurrentY, Canvas.Width - 70dip, ControlHeight)	
	
	IVLeftArrowDay.Initialize("IVLeftArrowDay")
	Canvas.AddView(IVLeftArrowDay, 0, CurrentY, ControlHeight, ControlHeight)
	RenderSVG(IVLeftArrowDay, "i-left-arrow.svg", 1)
		
	IVRightArrowDay.Initialize("IVRightArrowDay")
	Canvas.AddView(IVRightArrowDay, Canvas.Width - 30dip, CurrentY, ControlHeight, ControlHeight)
	RenderSVG(IVRightArrowDay, "i-right-arrow.svg", 1)	
	
	CurrentY = CurrentY + ControlHeight + ControlGapV
	
	' MONTH
	' Add Label to Show Current Date
	LblMonth.Initialize("LblMonth")
	LblMonth.Text = GetMonthName(DateTime.GetMonth(CurrentStamp))
	LblMonth.TextSize = CDDatePickerStorage.FontSize
	LblMonth.TextColor = Props.Get("ControlFontColor")
	
	LblMonth.Gravity = Gravity.CENTER
	
	Canvas.AddView(LblMonth, 35dip, CurrentY, Canvas.Width - 70dip, ControlHeight)	
	
	IVLeftArrowMonth.Initialize("IVLeftArrowMonth")
	Canvas.AddView(IVLeftArrowMonth, 0, CurrentY, ControlHeight, ControlHeight)
	RenderSVG(IVLeftArrowMonth, "i-left-arrow.svg", 1)
		
	IVRightArrowMonth.Initialize("IVRightArrowMonth")
	Canvas.AddView(IVRightArrowMonth, Canvas.Width - 30dip, CurrentY, ControlHeight, ControlHeight)
	RenderSVG(IVRightArrowMonth, "i-right-arrow.svg", 1)
	
	CurrentY = CurrentY + ControlHeight + ControlGapV	
	
	UnfoldedHeight = CurrentY	
	CDDatePickerStorage.PickerHeight = UnfoldedHeight	
	
	If((Props.ContainsKey("ControlFont")) And (Props.Get("ControlFont") <> "")) Then
		Try
			Dim Const StandardFont As Typeface = Typeface.CreateNew(Typeface.LoadFromAssets(Props.Get("ControlFont")), Typeface.STYLE_BOLD)				
			
			LblDate.Typeface = StandardFont
			LblMonth.Typeface = StandardFont
			LblDay.Typeface = StandardFont			
		Catch
			Log(LastException)
		End Try
	End If	
	
	MyTimer.Initialize("MyTimer", 500)	
	MyTimer.Enabled = True
End Sub

Sub GetDayString(DateStamp As Long) As String
	DateTime.DateFormat = "EEE d"
	Dim Result As String = DateTime.Date(DateStamp)
	
	Return Result
End Sub

Public Sub GetBase As Panel
	Return Canvas
End Sub

Private Sub GetMonthName(MonthNo As Int)
	Dim MonthStr As String = ""
	
	Select MonthNo
		Case 1
			MonthStr = "January"
		Case 2
			MonthStr = "Februrary"
		Case 3
			MonthStr = "March"
		Case 4
			MonthStr = "April"
		Case 5
			MonthStr = "May"
		Case 6
			MonthStr = "June"															
		Case 7
			MonthStr = "July"
		Case 8
			MonthStr = "August"
		Case 9
			MonthStr = "September"
		Case 10
			MonthStr = "October"
		Case 11
			MonthStr = "November"
		Case 12
			MonthStr = "December"			
	End Select
	
	Return MonthStr
End Sub

Sub LblDate_Click
	HandleFolding
End Sub

Sub IVImageView_Click
	HandleFolding
End Sub

Sub HandleFolding
	If(Unfolded = False) Then
		Unfolded = True
		CurrentHeight = UnfoldedHeight
		RenderSVG(IVDownArrow, "i-up-arrow.svg", 1)
		
	Else
		Unfolded = False
		CurrentHeight = InitialHeight
		RenderSVG(IVDownArrow, "i-down-arrow.svg", 1)
	End If
	
	Canvas.Height = CurrentHeight
	
	If(SubExists(CallBack, EventName & "_Folding")) Then
		CallSub2(CallBack, EventName & "_Folding", Unfolded)
	End If		
End Sub

Sub IVRightArrowMonth_Click
	CurrentStamp = DateTime.Add(CurrentStamp, 0, 1, 0)
	UpdateDisplay
End Sub

Sub IVLeftArrowMonth_Click
	CurrentStamp = DateTime.Add(CurrentStamp, 0, -1, 0)
	UpdateDisplay
End Sub

Sub IVRightArrowDay_Click
	CurrentStamp = DateTime.Add(CurrentStamp, 0, 0, 1)
	UpdateDisplay
End Sub

Sub IVLeftArrowDay_Click
	CurrentStamp = DateTime.Add(CurrentStamp, 0, 0, -1)
	UpdateDisplay
End Sub

Public Sub UpdateDisplay
	LblMonth.Text = GetMonthName(DateTime.GetMonth(CurrentStamp))
	LblDay.Text = GetDayString(CurrentStamp)
	Log("Setting to stamp: " & DateTime.Date(CurrentStamp))
	
	DateTime.DateFormat = DateFormatStr
	LblDate.Text = DateTime.Date(CurrentStamp)
	
	If(SubExists(CallBack, EventName & "_DateChanged")) Then
		CallSub2(CallBack, EventName & "_DateChanged", CurrentStamp)
	End If
End Sub

' **************************************** SVG UTILITY METHODS ********************************
' Renders an SVG image into an ImageView object.  The height ratio is proportional to the width. 
' E.g. a ratio of 0.5 would mean the image is half as high as the width.
Private Sub RenderSVG(target As ImageView, svgFile As String, heightRatio As Float)
	target.Height = CalculateHeight(target.Width, heightRatio)
	
	Dim tCanvas As Canvas
	tCanvas.Initialize(target)
	tCanvas.DrawColor(Colors.Transparent)
	'tCanvas.DrawColor(Colors.Magenta)
	
	Dim SVG As ioxSVG
	SVG.Initialize(svgFile)	
	SVG.DocumentWidth = target.Width
	SVG.DocumentHeight = target.Height
	SVG.setDocumentPreserveAspectRatio(SVG.SVGAlignment.XMaxYMax, SVG.SVGScale.NullScale)
	'SVG.setDocumentPreserveAspectRatio(SVG.SVGAlignment.NullAlignment, SVG.SVGScale.NullScale)
	SVG.RenderToCanvas(tCanvas)
End Sub

Private Sub RenderSVGFill(target As ImageView, svgFile As String)	
	Dim tCanvas As Canvas
	tCanvas.Initialize(target)
	tCanvas.DrawColor(Colors.Transparent)
	'tCanvas.DrawColor(Colors.Magenta)
	
	Dim SVG As ioxSVG
	SVG.Initialize(svgFile)	
	SVG.DocumentWidth = target.Width
	SVG.DocumentHeight = target.Height
	SVG.setDocumentPreserveAspectRatio(SVG.SVGAlignment.None, SVG.SVGScale.NullScale)
	SVG.RenderToCanvas(tCanvas)
End Sub

' Calculates width x ratio
Private Sub CalculateHeight(Width As Int, Ratio As Float) As Float
	Dim height As Float = Width * Ratio
	Return height
End Sub

Public Sub SetDate(Ticks As Long)
	CurrentStamp = Ticks
	UpdateDisplay	
End Sub

Sub MyTimer_Tick
	'Log("Tick: " & CDDatePickerStorage.SetTimestamp)
	'Log("Tick: " & DateTime.Date(CDDatePickerStorage.SetTimestamp))
	If(CDDatePickerStorage.SetTimestamp > 0) Then
		CurrentStamp = CDDatePickerStorage.SetTimestamp
		CDDatePickerStorage.SetTimestamp = 0
		UpdateDisplay
	End If
End Sub