﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@

Sub Class_Globals
	Private CallbackObject As Object
	Private Canvas As Panel	
	Private CanvasReady As Boolean = False
	Private LblHeading As Label
	Private StrUtils As StringUtils
	
	Private DateStamp As Long
	Private DateYear As Int
	Private DateMonth As Int
	Private DateDay As Int
	Private DateIsLeapYear As Boolean
	
	Private Const ControlGapV As Int = 10dip
	Private Const ControlGapH As Int = 8dip	
	Private Const MarginX As Int = 5dip
	Private Const MarginY As Int = 5dip	
	
	Public Heading As String = "Select a Date"
	Public HeadingSize As Int = 12dip
	Public HeadingColor As Int = Colors.DarkGray
	
	Public DayHeadingColor As Int = Colors.Cyan	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Callback As Object, BGColor As Int)
	SetDate(DateTime.Now)
	
	CallbackObject = Callback
	Canvas.Initialize("Canvas")
	Canvas.Color = BGColor
	Canvas.Visible = False
	
	CanvasReady = False
	LblHeading.Initialize("LblHeading")
End Sub

Public Sub GetCanvas As View
	Return Canvas
End Sub

Public Sub SetDate(Timestamp As Long)
	DateStamp = Timestamp
	DateYear = DateTime.GetYear(DateStamp)
	DateMonth = DateTime.GetMonth(DateStamp)
	DateDay = DateTime.GetDayOfMonth(DateStamp)
	
	' Determine if Leap year
	If(DateYear Mod 4 = 0) Then
		If(DateYear Mod 100 = 0) Then
			If(DateYear Mod 400 = 0) Then	
				DateIsLeapYear = True	
			Else
				DateIsLeapYear = False
			End If
		Else
			DateIsLeapYear = True
		End If	
	Else
		DateIsLeapYear = False
	End If
End Sub

Private Sub SetupCanvas

	Dim FullWidth As Int = 100%x - (MarginX * 2)

	Dim ControlHeight As Int = 0
	
	Dim CurrentX As Int = MarginX
	Dim CurrentY As Int = MarginX
	
	Try
		' Add the main heading to the canvas
		LblHeading.Text = "Select a Date"
		LblHeading.TextColor = HeadingColor
		LblHeading.TextSize = HeadingSize
		
		Canvas.AddView(LblHeading, CurrentX, CurrentY, FullWidth, 30)
		ControlHeight = StrUtils.MeasureMultilineTextHeight(LblHeading, LblHeading.Text)
		LblHeading.Height = ControlHeight
		
		CurrentY = CurrentY + LblHeading.Height + ControlGapV	
		
		' Add The Mon to Sun Labels
		Dim Days(7) As String
		Dim DayLabels(7) As Label
		Days(0) = "MON"
		Days(1) = "TUE"
		Days(2) = "WED"
		Days(3) = "THU"
		Days(4) = "FRI"
		Days(5) = "SAT"
		Days(6) = "SUN"
		
		Dim DayNo As Int = 0
		Dim DayFontSize As Int = DetermineFontSize
		
		Do While(DayNo < 7)
			DayLabels(DayNo).Initialize("")
			DayLabels(DayNo).Text = Days(DayNo)
			DayLabels(DayNo).TextColor = DayHeadingColor
			DayLabels(DayNo).TextSize = DayFontSize	
			DayLabels(DayNo).Color = Colors.DarkGray		
			DayLabels(DayNo).Gravity = Gravity.CENTER

			Canvas.AddView(DayLabels(DayNo), CurrentX, CurrentY, 40dip, 30dip)
			CurrentX = CurrentX + 40dip + ControlGapH
			
			DayNo = DayNo + 1
		Loop
			
	Catch
		Log(LastException)
	End Try


	Canvas.Visible = False
	CanvasReady = True
End Sub

' Figure out the maximum possible font size that will fit in the label
Private Sub DetermineFontSize As Int
	Dim Lbl As Label
	Dim DateWidth As Int = (100%x - (2 * MarginX) - (7 * ControlGapH)) / 7
	Lbl.Initialize("")
	Lbl.Text = "ZZZ"
	Lbl.TextSize = 10
	Lbl.Visible = False
	
	Canvas.AddView(Lbl, 0, 0, DateWidth, 20dip)
	
	Dim Fits As Boolean = True
	Dim TxtHeight As Int = StrUtils.MeasureMultilineTextHeight(Lbl, Lbl.Text)
	Dim CurrentSize As Int = 10
	
	Do While(Fits)
		Dim ThisHeight As Int = StrUtils.MeasureMultilineTextHeight(Lbl, Lbl.Text)
		Dim Diff As Int = ThisHeight - TxtHeight
		Log("Height: " & ThisHeight)
		If(Diff > 20) Then
			Fits = False
			Return CurrentSize - 1
		End If
		
		CurrentSize = CurrentSize + 1
		Lbl.TextSize = CurrentSize		
	Loop	
	
	Return 0
End Sub

Public Sub Show
	Canvas.RemoveAllViews
	CanvasReady = False
	
	Canvas.Visible = True
	
	If(CanvasReady = False) Then
		SetupCanvas
	End If
End Sub